import React from 'react'
import styled from 'styled-components'
import { Icon, Button } from '@pubsweet/ui'
import { th, darken, lighten } from '@pubsweet/ui-toolkit'
import * as PDFJS from 'pdfjs-dist/webpack.js'
import { A } from '../ui'
import Viewer from './Viewer'

const OuterContainer = styled.div`
  max-width: 100%;
  background-color: ${th('colorBackgroundHue')};
  border: ${th('borderWidth')} ${th('borderStyle')} ${th('colorBorder')};
  position: relative;
  height: 100vh;
  margin: 0;
  overflow: hidden;
  &.full {
    position: absolute;
    top: 0;
    right: 0;
    left: 0;
    z-index: 2;
  }
`
const Toolbar = styled.div`
  height: calc(${th('gridUnit')} * 3);
  background-color: ${darken('colorBackgroundHue', 1)};
  box-shadow: 0 0 8px ${darken('colorBackgroundHue', 30)};
  position: relative;
  z-index: 2;
  display: flex;
  justify-content: space-between;
`
const Zoom = styled(Button)`
  display: inline-flex;
  vertical-align: top;
  margin: 0;
  height: calc(${th('gridUnit')} * 3);
  padding: 0;
  min-width: 0;
  line-height: calc(${th('gridUnit')} * 3);
  border: ${th('borderWidth')} ${th('borderStyle')} ${th('colorBorder')};
  background-color: ${th('colorBackgroundHue')};
  font-weight: normal;
  color: ${th('colorText')};
  font-size: ${th('fontSizeBaseSmall')}
  &:hover {
    background-color: ${lighten('colorBackgroundHue', 4)};
  }
`
const Info = styled.span`
  display: inline-block;
  vertical-align: top;
  margin: 0 ${th('gridUnit')};
  height: calc(${th('gridUnit')} * 3);
  line-height: calc(${th('gridUnit')} * 3);
  font-size: ${th('fontSizeBaseSmall')};
`
class PDFViewer extends React.Component {
  constructor(props) {
    super(props)
    this.state = {
      pdf: null,
      text: [],
      page: 1,
      scale: 1,
      screenSize: 'normal',
    }
    this.scaleToFit = this.scaleToFit.bind(this)
    this.zoomIn = this.zoomIn.bind(this)
    this.zoomOut = this.zoomOut.bind(this)
    this.adjustScreen = this.adjustScreen.bind(this)
    this.countPages = this.countPages.bind(this)
    this.deliverText = this.deliverText.bind(this)
    this.viewerRef = React.createRef()
  }
  componentDidMount() {
    PDFJS.getDocument(this.props.url).then(pdf => {
      this.setState({ pdf })
    })
  }
  scaleToFit() {
    const { scale } = this.state
    const desiredWidth = this.viewerRef.current.offsetWidth
    const viewBoxWidth = this.viewerRef.current.getElementsByClassName(
      'page',
    )[0].offsetWidth
    const newScale =
      parseInt((desiredWidth * scale) / viewBoxWidth / 0.25, 10) * 0.25
    this.setState({ scale: newScale })
  }
  zoomIn() {
    const { scale } = this.state
    if (scale < 5) {
      if (scale >= 2) {
        this.setState({ scale: scale + 1 })
      } else {
        this.setState({ scale: scale + 0.25 })
      }
    }
  }
  zoomOut() {
    const { scale } = this.state
    if (scale > 0.5) {
      if (scale >= 3) {
        this.setState({ scale: scale - 1 })
      } else {
        this.setState({ scale: scale - 0.25 })
      }
    }
  }
  adjustScreen() {
    const { screenSize } = this.state
    this.setState({ screenSize: screenSize === 'normal' ? 'full' : 'normal' })
  }
  countPages() {
    const { top } = this.viewerRef.current.getBoundingClientRect()
    const pages = this.viewerRef.current.getElementsByClassName('pdf-page')
    const topPages = Array.prototype.filter.call(
      pages,
      page => page.getBoundingClientRect().top <= top,
    )
    if (topPages.length > 0) {
      const page = topPages[topPages.length - 1].dataset.pageNumber
      if (this.state.page !== page) {
        this.setState({ page })
      }
    }
  }
  deliverText(pageText) {
    if (this.props.textContent || this.props.loaded) {
      this.setState({ text: [...this.state.text, pageText] }, () => {
        const { text, pdf } = this.state
        if (text.length === pdf._pdfInfo.numPages) {
          const sorted = text.sort((a, b) => a.page - b.page)
          if (this.props.textContent) {
            this.props.textContent(sorted.map(p => p.text).join(' '))
          }
          if (this.props.loaded) {
            this.props.loaded(true)
          }
        }
      })
    }
  }
  render() {
    const { pdf, scale, page, screenSize } = this.state
    const { url } = this.props
    return (
      <OuterContainer
        className={`pdf-viewer ${screenSize}`}
        data-pages={pdf && pdf._pdfInfo && pdf._pdfInfo.numPages}
        ref={this.viewerRef}
      >
        <Toolbar>
          {pdf && pdf._pdfInfo && (
            <Info>
              Page {page} of {pdf._pdfInfo.numPages}
            </Info>
          )}
          <div>
            <Zoom
              onClick={this.scaleToFit}
              style={{ padding: '0 4px', marginRight: '4px' }}
            >
              Scale to fit
            </Zoom>
            <Zoom onClick={this.zoomOut}>
              <Icon size={2}>minus</Icon>
            </Zoom>
            <Zoom onClick={this.zoomIn}>
              <Icon size={2}>plus</Icon>
            </Zoom>
            <Info>{scale * 100}%</Info>
          </div>
          <div>
            <A href={url} target="_blank">
              <Zoom>
                <Icon size={2}>download</Icon>
              </Zoom>
            </A>
            <Zoom onClick={this.adjustScreen} style={{ padding: '0 4px' }}>
              {screenSize === 'normal' ? 'Full screen' : 'Exit full screen'}
              <Icon size={2}>
                {screenSize === 'normal' ? 'maximize-2' : 'minimize-2'}
              </Icon>
            </Zoom>
          </div>
        </Toolbar>
        {pdf && (
          <Viewer
            onScroll={this.countPages}
            pdf={pdf}
            scale={scale}
            textContent={this.deliverText}
          />
        )}
      </OuterContainer>
    )
  }
}

export default PDFViewer
